/* main.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "main.h"

struct screen_wh {
    GtkWidget *width_entry;
    GtkWidget *height_entry;
    GtkWidget *ok_btn;
};

gboolean have_to_reboot = FALSE;

/* screensize combo handling */
static void
on_swh_changed (GtkWidget *widget, gpointer data)
{
    struct screen_wh *swh = (struct screen_wh *) data;
    gchar *w = g_strdup (gtk_entry_get_text (GTK_ENTRY (swh->width_entry)));
    gchar *h = g_strdup (gtk_entry_get_text (GTK_ENTRY (swh->height_entry)));

    if (!strcmp ("", w) || (strspn (w, "0123456789") != strlen (w)))
        gtk_widget_set_sensitive (swh->ok_btn, FALSE);
    else if (!strcmp ("", h) || (strspn (h, "0123456789") != strlen (h)))
        gtk_widget_set_sensitive (swh->ok_btn, FALSE);
    else
        gtk_widget_set_sensitive (swh->ok_btn, TRUE);

    g_free (w);
    g_free (h);
}

static const gchar *
screensize_popup ()
{
    GtkWidget *d;
    GtkWidget *label, *table;
    struct screen_wh *swh;
    gchar *w = NULL;
    gchar *h = NULL;

    swh = g_new (struct screen_wh, 1);

    d = gtk_dialog_new_with_buttons (_("Screen size"),
        GTK_WINDOW (dialog->dialog),
        GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
        NULL);

    table = gtk_table_new (2, 2, FALSE);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (d)->vbox), table,
        TRUE, TRUE, 0);
    gtk_widget_show (table);

    label = gtk_label_new (_("Width: "));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
        GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 2, 2);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_widget_show (label);

    label = gtk_label_new (_("Height: "));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
        GTK_FILL | GTK_SHRINK, GTK_FILL | GTK_SHRINK, 2, 2);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_widget_show (label);

    swh->width_entry = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), swh->width_entry, 1, 2, 0, 1,
        GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 2, 2);
    gtk_widget_show (swh->width_entry);

    swh->height_entry = gtk_entry_new ();
    gtk_table_attach (GTK_TABLE (table), swh->height_entry, 1, 2, 1, 2,
        GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 2, 2);
    gtk_widget_show (swh->height_entry);

    swh->ok_btn = gtk_dialog_add_button (
        GTK_DIALOG (d), GTK_STOCK_OK, GTK_RESPONSE_OK);
    gtk_widget_set_sensitive (swh->ok_btn, FALSE);

    g_signal_connect (G_OBJECT (swh->width_entry), "changed",
        G_CALLBACK (on_swh_changed), (gpointer) swh);
    g_signal_connect (G_OBJECT (swh->height_entry), "changed",
        G_CALLBACK (on_swh_changed), (gpointer) swh);

    w = (gchar *) g_object_get_data (G_OBJECT (dialog->screen_combo), "width");
    h = (gchar *) g_object_get_data (G_OBJECT (dialog->screen_combo), "height");
    if (w)
        gtk_entry_set_text (GTK_ENTRY (swh->width_entry), w);
    if (h)
        gtk_entry_set_text (GTK_ENTRY (swh->height_entry), h);

    if (gtk_dialog_run (GTK_DIALOG (d)) == GTK_RESPONSE_OK)
    {
        w = g_strdup (gtk_entry_get_text (GTK_ENTRY (swh->width_entry)));
        h = g_strdup (gtk_entry_get_text (GTK_ENTRY (swh->height_entry)));

        gtk_widget_destroy (d);

        if (!strcmp (w, "") || !strcmp (h, ""))
        {
            g_free (w);
            g_free (h);

            return "";
        }

        g_object_set_data (G_OBJECT (dialog->screen_combo),
            "width", (gpointer) w);
        g_object_set_data (G_OBJECT (dialog->screen_combo),
            "height", (gpointer) h);
        return  g_strdup_printf ("%sx%s", w, h);
    } else {
        gtk_widget_destroy (d);
    }

    return "";
}

/* callbacks */
static void
run_help (GtkWidget *widget, gpointer data)
{
    run_user_cmd ("mousepad /cdrom/readme.txt", TRUE);
}

static void
delete_event (GtkWidget *widget, GdkEvent *event, gpointer data)
{
    /* restore the initial kbd conf */
    restore_kbd_config ();
    gtk_main_quit ();
}

static void
kbd_btn_clicked (GtkWidget *widget, gpointer data)
{
    layout_dialog_run ();
}

static void
tz_btn_clicked (GtkWidget *widget, gpointer data)
{
    tz_dialog_run ();
}

static void
locale_btn_clicked (GtkWidget *widget, gpointer data)
{
    locale_dialog_run ();
}

static void
screen_combo_changed (GtkComboBox *combo, gpointer data)
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    gchar *tmp = NULL;
    const gchar *ret;

    model = gtk_combo_box_get_model (GTK_COMBO_BOX (dialog->screen_combo));
    gtk_combo_box_get_active_iter (GTK_COMBO_BOX (dialog->screen_combo), &iter);
    gtk_tree_model_get (model, &iter, COL_SCREEN_CODE, &tmp, -1);

    if (!strcmp ("custom", tmp))
    {
        g_free (tmp);
        ret = screensize_popup ();
        set_screensize (ret);
    }
}

static void
on_nm_check_toggled (GtkWidget *widget, gpointer data)
{
    gtk_widget_set_sensitive (GTK_WIDGET (data),
        !gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)));
}

static void
change_pwd (GtkWidget *w, gpointer data)
{
    gchar *tmp = get_lock_passwd ();
    if (tmp)
    {
        g_free (dialog->lock_pwd);
        dialog->lock_pwd = tmp;
    }
}

static void
lock_toggled (GtkWidget *btn, gpointer data)
{
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (btn))) {
        if (!dialog->lock_pwd) {
            gchar *tmp = get_lock_passwd ();
            if (tmp) {
                gtk_widget_set_sensitive (dialog->vbox_lock,
                             TRUE);
                g_free (dialog->lock_pwd);
                dialog->lock_pwd = tmp;
            } else {
                gtk_widget_set_sensitive (dialog->vbox_lock,
                             FALSE);
                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                                 (btn), FALSE);
            }
        } else
            gtk_widget_set_sensitive (dialog->vbox_lock, TRUE);
    } else
        gtk_widget_set_sensitive (dialog->vbox_lock, FALSE);
}

static void
save_config (GtkWidget *widget, gpointer data)
{
    const gchar *timezone = get_tz_value ();
    char *command =
        g_strconcat ("sh ", LIBEXECDIR, "/apply-tz.sh ", timezone, NULL);
    system (command);
    g_free (command);

    write_config ();

    if (get_reboot_needed ())
    {
        /* destroy the main window */
        gtk_widget_destroy (dialog->dialog);
        have_to_reboot = warn_about_changes ();
    }

    gtk_main_quit ();
}

static void
populate_screen_combo (GtkListStore *store)
{
    GtkTreeIter iter;
    int i = 0;
    gboolean l3 = is_linutop_3();

    while (resolutions[i].code != NULL) {
        if (!strcmp ("custom", resolutions[i].code) && !l3)
        {
            i++;
            continue;
        }

        gtk_list_store_append (store, &iter);
        gtk_list_store_set (store, &iter,
            COL_SCREEN_CODE, resolutions[i].code,
            COL_SCREEN_DISPLAY, resolutions[i].display,
            -1);
        i++;
    }
}

static void
create_dialog ()
{
    GtkWidget *label, *table, *vbox, *hbox, *notebook, *vbox2;
    GtkWidget *asrun_btn, *backup_btn, *format_btn, *image, *frame;
    GtkWidget *net_btn, *lock_pwd_btn;
    GtkWidget *advanced_vbox, *security_vbox;
    GtkWidget *mnt_hbox;
    GtkWidget *lock_w, *hr;
    GtkWidget *about, *aboutsysinfo;
    GtkWidget *energy;
    GtkWidget *vnc_btn;
    GtkWidget *nm_vbox, *nm_hbox;
    GtkListStore *screen_store;
    GtkCellRenderer *renderer;
    gboolean nm_is_there = FALSE;

    set_is_laptop ();

    gtk_window_set_default_icon_name ("preferences-desktop");

    dialog->dialog = gtk_dialog_new ();
    gtk_window_set_title (GTK_WINDOW (dialog->dialog), _("Linutop Setup"));
    gtk_window_set_resizable (GTK_WINDOW (dialog->dialog), FALSE);
    gtk_window_set_position (GTK_WINDOW (dialog->dialog), GTK_WIN_POS_CENTER);

    notebook = gtk_notebook_new ();
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dialog->dialog)->vbox),
              notebook);
    gtk_widget_show (notebook);

    /*
     * General tab
     */
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox);

    label = gtk_label_new (_("General"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

    table = gtk_table_new (2, 2, FALSE);
    gtk_widget_show (table);

    /* local settings */
    frame = xfce_framebox_new (_("Local settings"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), table);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    label = gtk_label_new (_("Language: "));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL,
             GTK_SHRINK, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_widget_show (label);
    dialog->locale_btn = gtk_button_new ();
    gtk_table_attach (GTK_TABLE (table), dialog->locale_btn, 1, 2, 0, 1,
             GTK_FILL | GTK_EXPAND, GTK_SHRINK, 0, 0);
    gtk_widget_show (dialog->locale_btn);

    label = gtk_label_new (_("Keyboard layout: "));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL,
             GTK_SHRINK, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_widget_show (label);
    dialog->kbd_btn = gtk_button_new ();
    gtk_table_attach (GTK_TABLE (table), dialog->kbd_btn, 1, 2, 1, 2,
             GTK_FILL | GTK_EXPAND, GTK_SHRINK, 0, 0);
    gtk_widget_show (dialog->kbd_btn);

    label = gtk_label_new (_("Timezone: "));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL,
             GTK_SHRINK, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_widget_show (label);
    dialog->tz_btn = gtk_button_new ();
    gtk_table_attach (GTK_TABLE (table), dialog->tz_btn, 1, 2, 2, 3,
             GTK_FILL | GTK_EXPAND, GTK_SHRINK, 0, 0);
    gtk_widget_show (dialog->tz_btn);

    /* Display settings */
    hbox = gtk_hbox_new (FALSE, 0);
    gtk_widget_show (hbox);

    label = gtk_label_new (_("Screen size:"));
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_widget_show (label);

    screen_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
    dialog->screen_combo = gtk_combo_box_new_with_model (
        GTK_TREE_MODEL (screen_store));
    renderer = gtk_cell_renderer_text_new ();
    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (dialog->screen_combo),
        renderer, TRUE);
    gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (dialog->screen_combo),
        renderer, "text", COL_SCREEN_DISPLAY);
    gtk_box_pack_start (GTK_BOX (hbox), dialog->screen_combo, TRUE, TRUE, 2);
    populate_screen_combo (screen_store);
    gtk_combo_box_set_active (GTK_COMBO_BOX (dialog->screen_combo), 0);
    gtk_widget_show (dialog->screen_combo);

    frame = xfce_framebox_new (_("Display settings"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), hbox);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    /* Autostarted app */
    vbox2 = gtk_vbox_new (FALSE, 2);
    gtk_widget_show (vbox2);

    asrun_btn = new_run_cmd_button ("xfce4-autostart-editor",
                                    _("Configure autostarted applications"),
                                    "appstarter", TRUE);
    gtk_widget_show (asrun_btn);

    gtk_box_pack_start (GTK_BOX (vbox2), asrun_btn, FALSE, FALSE, 2);
    gtk_widget_show (asrun_btn);

    dialog->next_time =
        gtk_check_button_new_with_label (_("Show this dialog at each boot"));
    gtk_box_pack_start (GTK_BOX (vbox2), dialog->next_time, FALSE, FALSE, 2);
    gtk_widget_show (dialog->next_time);

    frame = xfce_framebox_new (_("Autostarted application"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    /*
     * Network tab
     */
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox);

    label = gtk_label_new (_("Network"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);

    /* Network settings */
    nm_vbox = gtk_vbox_new (FALSE, 2);
    nm_hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (nm_vbox), nm_hbox, FALSE, FALSE, 2);

    dialog->nm_chk = gtk_check_button_new_with_label (
       _("Enable Network Manager"));
    gtk_box_pack_start (GTK_BOX (nm_hbox), dialog->nm_chk, FALSE, FALSE, 2);

    image = gtk_image_new_from_icon_name ("nm-signal-75", GTK_ICON_SIZE_SMALL_TOOLBAR);
    gtk_box_pack_start (GTK_BOX (nm_hbox), image, FALSE, FALSE, 2);

    nm_is_there = (g_access ("/etc/init.d/NetworkManager", X_OK) == 0);
    if (!nm_is_there)
        nm_is_there = (g_access ("/etc/init.d/network-manager", X_OK) == 0);
    if (!nm_is_there)
        gtk_widget_set_sensitive (nm_hbox, FALSE);

    gtk_widget_show_all (nm_vbox);
    gtk_widget_show (dialog->nm_chk);

    net_btn = new_run_cmd_button ("network", _("Configure your network"),
                                  "network-admin", FALSE);
    gtk_box_pack_start (GTK_BOX (nm_vbox), net_btn, FALSE, FALSE, 2);
    gtk_widget_show (net_btn);

    frame = xfce_framebox_new (_("Network settings"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);

    xfce_framebox_add (XFCE_FRAMEBOX (frame), nm_vbox);
    gboolean usable = (gboolean) g_object_get_data (G_OBJECT (net_btn), "usable");
    if (usable && nm_is_there)
        g_signal_connect (G_OBJECT (dialog->nm_chk), "toggled",
            G_CALLBACK (on_nm_check_toggled), (gpointer) net_btn);

    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    /* vnc */
    vnc_btn = new_run_cmd_button ("gnome-remote-desktop",
                                  _("Configure the remote access service"),
                                  "gx11vnc", TRUE);
    gtk_widget_show (vnc_btn);

    frame = xfce_framebox_new (_("Remote access"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vnc_btn);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    /* samba */
    mnt_hbox = create_mount_gui ();
    frame = xfce_framebox_new (_("Windows shares"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), mnt_hbox);
    gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    /*
     * advanced tab
     */
    advanced_vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (advanced_vbox);

    /* lbackup */
    backup_btn = new_run_cmd_button ("document-save", _("Backup/Restore"),
                                     "lbackup", FALSE);
    gtk_widget_show (backup_btn);

    frame = xfce_framebox_new (_("Backups"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), backup_btn);
    gtk_box_pack_start (GTK_BOX (advanced_vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    /* persistence */
    vbox2 = gtk_vbox_new (FALSE, 2);
    gtk_widget_show (vbox2);

    dialog->home_chk =
        gtk_radio_button_new_with_label (NULL,
                        (_
                         ("user settings only persistence")));
    gtk_box_pack_start (GTK_BOX (vbox2), dialog->home_chk, FALSE, FALSE, 0);
    gtk_widget_show (dialog->home_chk);
    dialog->full_chk =
        gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON
                             (dialog->home_chk),
                            _
                             ("full persistent system"));
    gtk_box_pack_start (GTK_BOX (vbox2), dialog->full_chk, FALSE, FALSE, 0);
    gtk_widget_show (dialog->full_chk);

    frame = xfce_framebox_new (_("Data persistence"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gtk_box_pack_start (GTK_BOX (advanced_vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    /* misc */
    vbox2 = gtk_vbox_new (FALSE, 2);
    gtk_widget_show (vbox2);

    dialog->nox_chk =
        gtk_check_button_new_with_label (_
                        ("Don't start Xorg (command line system)"));
    gtk_box_pack_start (GTK_BOX (vbox2), dialog->nox_chk, FALSE, FALSE, 0);
    gtk_widget_show (dialog->nox_chk);

    dialog->checkfs_chk =
        gtk_check_button_new_with_label (_
                        ("Check and try to fix filesystem errors"));
    gtk_box_pack_start (GTK_BOX (vbox2), dialog->checkfs_chk, FALSE, FALSE,
               0);
    gtk_widget_show (dialog->checkfs_chk);

    /* usb cleanup */
    format_btn = new_run_cmd_button ("edit-clear", _("Format a USB key (FAT32)"),
                                     "lusbclean", FALSE);
    gtk_box_pack_start (GTK_BOX (vbox2), format_btn, FALSE, FALSE, 2);
    gtk_widget_show (format_btn);

    frame = xfce_framebox_new (_("Misc"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gtk_box_pack_start (GTK_BOX (advanced_vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    label = gtk_label_new (_("Advanced"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), advanced_vbox, label);

    /*
     * security
     */
    security_vbox = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (security_vbox);

    vbox2 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox2);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox2), hbox, FALSE, FALSE, 2);
    gtk_widget_show (hbox);

    label = gtk_label_new (
        _("In locked mode the system cannot be altered\n"
          "(convenient for public access).\n"
          "A simple reboot will restore the system."));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);
    gtk_widget_show (label);

//    image = gtk_image_new_from_icon_name ("linutop-lock",
    image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_AUTHENTICATION,
        GTK_ICON_SIZE_DIALOG);
    gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, TRUE, 2);
    gtk_widget_show (image);

    dialog->lock_chk =
        gtk_check_button_new_with_label (_("Use the Linutop lock"));
    gtk_box_pack_start (GTK_BOX (vbox2), dialog->lock_chk, FALSE, FALSE, 2);
    gtk_widget_show (dialog->lock_chk);

    hr = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (vbox2), hr, FALSE, FALSE, 2);
    gtk_widget_show (hr);

    dialog->vbox_lock = gtk_vbox_new (FALSE, 2);
    gtk_widget_set_sensitive (dialog->vbox_lock, FALSE);
    gtk_widget_show (dialog->vbox_lock);

    lock_pwd_btn = gtk_button_new_with_label (_("Change password"));
    gtk_box_pack_start (GTK_BOX (dialog->vbox_lock), lock_pwd_btn,
               FALSE, FALSE, 2);
    gtk_widget_show (lock_pwd_btn);

    lock_w = lock_widget ();
    gtk_box_pack_start (GTK_BOX (dialog->vbox_lock), lock_w, FALSE, FALSE, 2);
    gtk_widget_show (lock_w);

    frame = xfce_framebox_new (_("Settings"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), dialog->vbox_lock);
    gtk_box_pack_start (GTK_BOX (vbox2), frame, FALSE, FALSE, 0);
    gtk_widget_show (frame);

    frame = xfce_framebox_new (_("Linutop lock"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox2);
    gtk_box_pack_start (GTK_BOX (security_vbox), frame, FALSE, FALSE, 4);
    gtk_widget_show (frame);

    label = gtk_label_new (_("Security"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), security_vbox, label);

    if (get_is_laptop ())
    {
        /* Energy */
        energy = get_energy_widget ();
        gtk_widget_show (energy);

        label = gtk_label_new (_("Energy"));
        gtk_notebook_append_page (GTK_NOTEBOOK (notebook), energy, label);
    }

    /* About */
    vbox2 = gtk_vbox_new (FALSE, 0);
    gtk_widget_show (vbox2);

    about = get_about_widget ();
    gtk_box_pack_start (GTK_BOX (vbox2), about, FALSE, FALSE, 2);

    hr = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (vbox2), hr, FALSE, FALSE, 2);
    gtk_widget_show (hr);

    aboutsysinfo = get_info_widget ();
    gtk_box_pack_start (GTK_BOX (vbox2), aboutsysinfo, FALSE, FALSE, 2);

    label = gtk_label_new (_("About"));
    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox2, label);

    /* buttons */
    hbox = GTK_DIALOG (dialog->dialog)->action_area;

    dialog->help_btn = gtk_button_new_from_stock (GTK_STOCK_HELP);
    gtk_box_pack_start (GTK_BOX (hbox), dialog->help_btn, TRUE, FALSE, 0);
    gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (hbox),
                                        dialog->help_btn, TRUE);
    gtk_widget_set_sensitive (dialog->help_btn,
                 (g_access (README, R_OK) == 0));
    gtk_widget_show (dialog->help_btn);

    dialog->quit_btn = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
    gtk_box_pack_start (GTK_BOX (hbox),dialog->quit_btn, TRUE, FALSE, 0);
    gtk_widget_show (dialog->quit_btn);

    dialog->save_btn = gtk_button_new_from_stock (GTK_STOCK_OK);
    gtk_box_pack_start (GTK_BOX (hbox), dialog->save_btn, TRUE, FALSE, 0);
    gtk_widget_set_sensitive (dialog->save_btn,
                 (g_access (CONFFILE, W_OK) == 0));
    gtk_widget_show (dialog->save_btn);

    read_config ();

    /* signals */
    g_signal_connect (G_OBJECT (dialog->kbd_btn), "clicked",
             G_CALLBACK (kbd_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (dialog->tz_btn), "clicked",
             G_CALLBACK (tz_btn_clicked), NULL);
    g_signal_connect (G_OBJECT (dialog->locale_btn), "clicked",
             G_CALLBACK (locale_btn_clicked), NULL);

    if (is_linutop_3 ())
        g_signal_connect (G_OBJECT (dialog->screen_combo), "changed",
             G_CALLBACK (screen_combo_changed), NULL);

    g_signal_connect (G_OBJECT (dialog->lock_chk), "toggled",
             G_CALLBACK (lock_toggled), NULL);
    g_signal_connect (G_OBJECT (lock_pwd_btn), "clicked",
             G_CALLBACK (change_pwd), NULL);

    g_signal_connect (G_OBJECT (dialog->help_btn), "clicked",
             G_CALLBACK (run_help), NULL);
    g_signal_connect (G_OBJECT (dialog->save_btn), "clicked",
             G_CALLBACK (save_config), NULL);
    g_signal_connect (G_OBJECT (dialog->quit_btn), "clicked",
             G_CALLBACK (delete_event), NULL);
    g_signal_connect (G_OBJECT (dialog->dialog), "delete_event",
             G_CALLBACK (delete_event), NULL);

    gtk_widget_show (dialog->dialog);
}

int
main (int argc, char *argv[])
{
    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
    gtk_init (&argc, &argv);
    gdk_notify_startup_complete ();
    int id = getuid ();
    if (id != 0) {
        xfce_warn (_("You don't appear to be root. Exiting."));
        exit (1);
    }

    dsp = XOpenDisplay (NULL);
    g_type_init ();
    engine = xkl_engine_get_instance (dsp);
    registry = xkl_config_registry_get_instance (engine);
    xkl_config_registry_load (registry);

    dialog = g_new0 (gui, 1);
    dialog->lock_pwd = NULL;

    create_dialog ();

    gtk_main ();

    g_free (dialog);

    if (have_to_reboot)
        system (REBOOTCMD);

    return 0;
}
