/**
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "config.h"

#include <stdio.h>
#include <string.h>

#include <libxfcegui4/libxfcegui4.h>

enum {
    TYPE_INFO = 0,
    TYPE_WARN,
    TYPE_ERR,
    TYPE_NOT_GOOD
};

int type = TYPE_INFO;

static gboolean get_type (const gchar *option_name, const gchar *value,
                          gpointer data, GError **error);

static GOptionEntry entries[] = {
    {"type", 't', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_CALLBACK,
     (gpointer) get_type,
     N_("Message type (can be 'info', 'warn' or 'error')"), NULL},
    {NULL}
};

static gboolean
get_type (const gchar *option_name, const gchar *value,
          gpointer data, GError **error)
{
    gboolean retval = TRUE;

    if (!strncmp ("info", value, sizeof (value)))
        type = TYPE_INFO;
    else if (!strncmp ("warn", value, sizeof (value)))
        type = TYPE_WARN;
    else if (!strncmp ("error", value, sizeof (value)))
        type = TYPE_ERR;
    else {
        type = TYPE_NOT_GOOD;
        retval = FALSE;
    }
    return retval;
}

int
main (int argc, gchar **argv)
{
    int i;

    xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
    gtk_init_with_args (&argc, &argv, _(""), entries, PACKAGE, NULL);

    if (type == TYPE_NOT_GOOD)
        return 1;

    if (argc == 1) {
        return 1;
    }

    gchar *message = strdup (argv[1]);
    gchar *tmp = NULL;
    for (i = 2; i < argc; i++)
    {
        tmp = g_strconcat (message, " ", argv[i], NULL);
        g_free (message);
        message = g_strdup (tmp);
        g_free (tmp);
    }

    switch (type) {
    case TYPE_INFO:
        xfce_info ("%s", message);
        break;
    case TYPE_WARN:
        xfce_warn ("%s", message);
        break;
    case TYPE_ERR:
        xfce_err ("%s", message);
        break;
    }

    g_free (message);

    return 0;
}
